      Subroutine ReadInputData
      Implicit NONE

      INCLUDE 'state.inc'

      Real*8    xb,yb,zb

      Integer*4 ix ,iy ,iz
      Integer*4 i,j,k,l
      Real*8    delta,aaa

      Real*8 DV
      Real*8 COEFF
      Real*8 sn_lte,n_lte(0:NLEVMAX)
      Real*8 de

      Real*8 tempdble
      Real*8 tempdblearray(0:NLEVMAX-1)

      Integer levels_in_table,number_of_tables
      Character*64 molecule,filename

      Real*8     xread(1:IXMAX+1)
      Real*8     yread(1:IYMAX+1)
      Real*8     zread(1:IZMAX+1)
      Real*8    vxread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)
      Real*8    vyread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)
      Real*8    vzread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)
      Real*8   rhoread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)
      Real*8    tkread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)
      Real*8 sigmaread(1:IXMAX+1,1:IYMAX+1,1:IZMAX+1)

c---- Main
c--- Read the NetCDF File
      
      call NetCDF_Read(INPUTCDFFILE,IXMAX+1, IYMAX+1, IZMAX+1,
     &                 XMAX,YMAX,ZMAX,
     &                 XMIN,YMIN,ZMIN,
     &                 xread, yread, zread,
     &                vxread,vyread,vzread,
     &               rhoread,tkread,sigmaread)

c---- Assign Read Data to Array used in the calculation
      deltax = (XMAX-XMIN)/Dble(IXMAX+1)
      deltay = (YMAX-YMIN)/Dble(IYMAX+1)
      deltaz = (ZMAX-ZMIN)/Dble(IZMAX+1)

      Do ix = 0,IXMAX
         x(ix) = (ix-IXMAX*0.5D0)*deltax
      End do
      
      Do iy = 0,IYMAX
         y(iy) = (iy-IYMAX*0.5D0)*deltay
      End do
      
      Do iz = 0,IZMAX
         z(iz) = (iz-IZMAX*0.5D0)*deltay
      End do

      Do iz = 0,IZMAX
         Do iy = 0,IYMAX
            Do ix = 0,IXMAX
               ndens0(0,ix,iy,iz) = rhoread(ix+1,iy+1,iz+1)
                   vx(  ix,iy,iz) =  vxread(ix+1,iy+1,iz+1)
                   vy(  ix,iy,iz) =  vyread(ix+1,iy+1,iz+1)
                   vz(  ix,iy,iz) =  vzread(ix+1,iy+1,iz+1)
                   tk(  ix,iy,iz) =  tkread(ix+1,iy+1,iz+1)
            inv_sigma(  ix,iy,iz) = 1.0D0/
     &                            sigmaread(ix+1,iy+1,iz+1)
            End do
         End do
      End do

      VXG   = 0.0D0
      VYG   = 0.0D0
      VZG   = 0.0D0
      DVMAX = 0.0D0
      Do iz = 0,IZMAX
         Do iy = 0,IYMAX
            Do ix = 0,IXMAX

               VXG = VXG + vx(ix,iy,iz)
               VYG = VYG + vy(ix,iy,iz)
               VZG = VZG + vz(ix,iy,iz)
               
               DVMAX = Max(DVMAX,
     &                  Sqrt(vx(ix,iy,iz)*vx(ix,iy,iz)
     &                      +vy(ix,iy,iz)*vy(ix,iy,iz)
     &                      +vz(ix,iy,iz)*vz(ix,iy,iz))
     &                      +3.0D0/inv_sigma(ix,iy,iz))

            End do
         End do
      End do
      DVMAX =  DVMAX*2.0D0

      aaa   = 1.0D0/Dble((IXMAX+1)*(IYMAX+1)*(IZMAX+1))
      VXG   = vxg *aaa
      VYG   = vyg *aaa
      VZG   = vzg *aaa

c---- Define Boundary Mesh
      
      x(-1)= x(0) - deltax
      y(-1)= y(0) - deltay
      z(-1)= z(0) - deltaz

      x(IXMAX+1)= x(IXMAX) + deltax
      y(IYMAX+1)= y(IYMAX) + deltay
      z(IZMAX+1)= z(IZMAX) + deltaz

c---- Define positions of cell interface.

      Do ix = 0,IXMAX+1
         xm(ix) = 0.5D0*(x(ix)+x(ix-1))
      End do
      Do iy = 0,IYMAX+1
         ym(iy) = 0.5D0*(y(iy)+y(iy-1))
      End do
      Do iz = 0,IZMAX+1
         zm(iz) = 0.5D0*(z(iz)+z(iz-1))
      End do

      RMAX = Sqrt(xm(0)*xm(0)+ym(0)*ym(0)+zm(0)*zm(0))

c---- Define PARAMETERS.
      HC = H*LC          !h * lc

      INV_PLANCK = 1.0D0/H
      INV_LC     = 1.0D0/LC

      TWOPI      = PI*PI
      INV_4PI    = 0.25D0/PI
      INV_SQRT_PI= 1.0D0/Sqrt(PI)

      INV_TREF   = 1.0D0/(KB*T_REF)
      INV_TBG    = 1.0D0/(KB * TBG)

      VCELL = deltax*deltay*deltaz   ! Cell volume
      INV_VCELL = 1.0D0/VCELL

c---- NS:DUMMY (NS=1)
      NL(NS)= NLEVMAX-1 !cal. max level number

      DO k=-1,NL(NS)+1
         DO j=-1,NL(NS)+1
            A(NS,j,k) = 0.0D0
            B(NS,j,k) = 0.0D0
            C(NS,j,k) = 0.0D0
         End do
      End do

c---- Collisional Coefficient Table

      filename = TABLENAME
      Open(Unit=10,File=filename,Status='OLD') 

      Read (10,*) molecule
      Write(*,*) molecule
      Read (10,*) XC
      Read (10,*) MU
      Read (10,*) BB
      Write(*,*) 'Abundance              = ',XC
      Write(*,*) 'Electric Dipole Moment = ',MU
      Write(*,*) 'Rotational Constant    = ',BB
      Read (10,*) levels_in_table
      Read (10,*) number_of_tables

      If (levels_in_table.ne.NLEVMAX) Then
         write(*,*) "Maximum level is incorrect!!"
         write(*,*) "Set NLEVMAX = ",levels_in_table,
     &              " (in parameter.inc ) and recompile the program."
         write(*,*) levels_in_table,NLEVMAX
         Stop
      Endif
      If (number_of_tables.ne.TABLEMAX) Then
         write(*,*) "The number of Level data is inconsistent!!"
         write(*,*) "Set TABLEMAX = ",number_of_tables,
     &              " and recompile the program."
         Stop
      Endif
      Read(10,*) 
      
      Do l=TABLEMIN,TABLEMAX
         
         Read(10,*) tklist(l)
         Read(10,*)          

         Do j=0,NLEVMAX-1
            Read(10,*) (tempdblearray(i),i=0,NLEVMAX-1)
            Do k = 0,NLEVMAX-1
               CTABLE(l,k,j) = tempdblearray(k)
            End do
         End do
         
      End do

      Close(Unit=10)

c---- Einstein A and B Coefficient.

      j = NL(NS)+1
      elev(j)=BB*j*(j+1.0D0)*H

      COEFF=32.D0/3.0D0 *(PI**4)*(MU**2)/(H**2)/LC

      Do j = 0,NL(NS)
         
         elev(j) = BB*j*(j+1.0D0)*H

         B(NS,j+1,j  )=COEFF*(j+1.0D0)/(2.0D0*j+3.0D0)
         B(NS,j  ,j+1)=(2.0D0*j+3.0D0)/(2.0D0*j+1.0D0)*B(NS,j+1,j)
         A(NS,j+1,j  )=16.D0*H*BB*((BB/LC)**2)*(j+1.0D0)**3*B(NS,j+1,j)

      End do

      A(NS,NL(NS)+1,NL(NS)) = 0.0D0
      B(NS,NL(NS)+1,NL(NS)) = 0.0D0
      B(NS,NL(NS),NL(NS)+1) = 0.0D0

c---- Define statistical weight. Will be given in data file.

      Do i = 0,NL(NS)
         gstat(i) = 2.0D0*i + 1
      End do

c---- Calculete LTE levele population of molecule.(J=0,1,...NL(NS))  (NS=1)
      Do iz = 0,IZMAX
         Do iy = 0,IYMAX
            Do ix = 0,IXMAX

               sn_lte=1.0D0
               n_lte(0) = 1.0D0
               DO j=1,NL(NS)
                  de=(elev(j)-elev(0))
                  n_lte(j) = (2.0D0*j+1.0D0)*Exp(-de/(KB*tk(ix,iy,iz)))
                  sn_lte = sn_lte + n_lte(j) 
               End do
               Do j = 0,NL(NS)
                  n_lte(j) = n_lte(j)/sn_lte
               End do


               ndens0(NS,ix,iy,iz)=ndens0(0,ix,iy,iz)*XC
               
               Do j = 0,NL(NS)

              ndens     (NS,j,ix,iy,iz)=ndens0(NS,  ix,iy,iz)*n_lte(j)
              ndens_ref (NS,j,ix,iy,iz)=ndens (NS,j,ix,iy,iz)
              ndens_prev(NS,j,ix,iy,iz)=ndens (NS,j,ix,iy,iz)

               End do

c---- Set Highest+1 and Lowest-1 =0
               ndens     (NS,NL(NS)+1,ix,iy,iz)=0.0D0
               ndens_ref (NS,NL(NS)+1,ix,iy,iz)=0.0D0
               ndens_prev(NS,NL(NS)+1,ix,iy,iz)=0.0D0
               ndens     (NS,      -1,ix,iy,iz)=0.0D0
               ndens_ref (NS,      -1,ix,iy,iz)=0.0D0
               ndens_prev(NS,      -1,ix,iy,iz)=0.0D0
c----

               Do j=-1,NL(NS)+1
                  ss     (j,ix,iy,iz)=0.0D0
                  ss_bk  (j,ix,iy,iz)=0.0D0
                  ss_prev(j,ix,iy,iz)=0.0D0
                  dss    (j,ix,iy,iz)=0.0D0
                  dss2   (j,ix,iy,iz)=0.0D0
                  ss_ref (j,ix,iy,iz)=0.0D0
               End do

            End do
         End do
      End do


      Return 
      End
